/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WizardController.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class is responsible for reacting to events generated by pushing any of
 * the three buttons, 'Next', 'Previous', and 'Cancel.' Based on what the
 * controller will update the model to show a new panel and reset the state of
 * the buttons as necessary.
 * @author skabore
 */
public class WizardController implements ActionListener {

    /**
     * The wizard.
     */
    private Wizard wizard;

    /**
     * This constructor accepts a reference to the Wizard component that created
     * it, which it uses to update the button components and access the
     * WizardModel.
     * @param wizard A callback to the Wizard component that created this
     *        controller.
     */
    public WizardController(final Wizard wizard) {
        this.wizard = wizard;
    }

    /**
     * Calling method for the action listener interface. This class listens for
     * actions performed by the buttons in the Wizard class, and calls methods
     * below to determine the correct course of action.
     * @param evt The ActionEvent that occurred.
     */
    public void actionPerformed(final ActionEvent evt) {
        if (evt.getActionCommand().equals(Wizard.CANCEL_BUTTON_ACTION_COMMAND)) {
            cancelButtonPressed();
        } else if (evt.getActionCommand().equals(Wizard.BACK_BUTTON_ACTION_COMMAND)) {
            backButtonPressed();
        } else if (evt.getActionCommand().equals(Wizard.NEXT_BUTTON_ACTION_COMMAND)) {
            nextButtonPressed();
        } else if (evt.getActionCommand().equals(Wizard.CONFIG_BUTTON_ACTION_COMMAND)) {
            configButtonPressed();
        } else if (evt.getActionCommand().equals(Wizard.STOP_BUTTON_ACTION_COMMAND)) {
            stopButtonPressed();
        }
    }

    /**
     * Set the current panel when 'cacel' button is pressed.
     */
    private void cancelButtonPressed() {
        deleteWorkDirectory();
        System.exit(Wizard.CANCEL_RETURN_CODE);
    }

    /**
     * Set the current panel when 'next' button is pressed.
     */
    private void nextButtonPressed() {
        WizardModel model = wizard.getModel();
        WizardPanelDescriptor descriptor = (WizardPanelDescriptor) model.getCurrentPanelDescriptor();
        Object nextPanelDescriptor = descriptor.getNextPanelDescriptor();
        if (nextPanelDescriptor instanceof WizardPanelDescriptor.FinishIdentifier) {
            System.exit(Wizard.FINISH_RETURN_CODE);
        } else {
            wizard.setCurrentPanel(nextPanelDescriptor);
        }
    }

    /**
     * Set the current panel when 'back' button is pressed.
     */
    private void backButtonPressed() {
        WizardModel model = wizard.getModel();
        WizardPanelDescriptor descriptor = (WizardPanelDescriptor) model.getCurrentPanelDescriptor();
        Object backPanelDescriptor = descriptor.getBackPanelDescriptor();
        wizard.setCurrentPanel(backPanelDescriptor);
    }

    /**
     * Set the current panel when 'config' button is pressed.
     */
    private void configButtonPressed() {
        WizardModel model = wizard.getModel();
        WizardPanelDescriptor descriptor = (WizardPanelDescriptor) model.getCurrentPanelDescriptor();
        Object configPanelDescriptor = descriptor.getConfigPanelDescriptor();
        wizard.setCurrentPanel(configPanelDescriptor);
    }

    /**
     * Set the current panel when 'Stop' button is pressed.
     */
    private void stopButtonPressed() {
        if (wizard.getStarter().jonasIsRunning()) {
            if (wizard.getStarter().isValidRequiredParameters()) {
                wizard.getStarter().getOutput().write("");
                new Thread() {

                    @Override
                    public void run() {
                        wizard.getStarter().getOutput().write("Stopping" + " JOnAS server...");
                        wizard.getStarter().stopJonas();
                        wizard.getStopButton().setEnabled(false);
                        wizard.getNextButton().setEnabled(true);
                    }
                }.start();
            } else {
                wizard.showMessage("Please set a valid JOnAS Root folder" + " before stopping server !");
            }
        }
    }

    /**
     * Reset buttons to panel rules.
     */
    public void resetButtonsToPanelRules() {
        WizardModel model = wizard.getModel();
        WizardPanelDescriptor descriptor = (WizardPanelDescriptor) model.getCurrentPanelDescriptor();
        model.setCancelButtonText(Wizard.CANCEL_TEXT);
        model.setBackButtonText(Wizard.BACK_TEXT);

        if (descriptor.getBackPanelDescriptor() != null) {
            model.setBackButtonEnabled(Boolean.TRUE);
        } else {
            model.setBackButtonEnabled(Boolean.FALSE);
        }

        if (descriptor.getNextPanelDescriptor() != null) {
            model.setNextFinishButtonEnabled(Boolean.TRUE);
        } else {
            model.setNextFinishButtonEnabled(Boolean.FALSE);
        }

        if (descriptor.getNextPanelDescriptor() instanceof WizardPanelDescriptor.FinishIdentifier) {
            model.setNextFinishButtonText(Wizard.FINISH_TEXT);
        } else {
            model.setNextFinishButtonText(Wizard.NEXT_TEXT);
        }
    }

    /**
     * Delete the working directory.
     */
    private void deleteWorkDirectory() {
        if (wizard.getStarter() != null && wizard.getStarter().getWorkdirectory() != null) {
            Utility.deleteDirectory(wizard.getStarter().getWorkdirectory());
        }
    }
}
