/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: WizardPanelDescriptor.java 20819 2011-02-02 16:00:53Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.wizard;

import java.awt.BorderLayout;
import java.awt.Component;

import javax.swing.JPanel;

/**
 * A base descriptor class used to reference a Component panel for the Wizard,
 * as well as provide general rules as to how the panel should behave.
 * @author skabore
 */
public class WizardPanelDescriptor {

    /**
     * The default panel identifier.
     */
    private static final String DEFAULT_PANEL_IDENTIFIER = "defaultPanelIdentifier";

    /**
     * Identifier returned by getNextPanelDescriptor() to indicate that this is
     * the last panel and the text of the 'Next' button should change to
     * 'Finish'.
     */
    public static final FinishIdentifier FINISH = new FinishIdentifier();

    /**
     * Identifier returned when 'Stop' button is pressed.
     */
    public static final String STOP_ID = "stop";

    /**
     * The wizard.
     */
    private Wizard wizard;

    /**
     * The target panel.
     */
    private Component targetPanel;

    /**
     * The panel identifier.
     */
    private Object panelIdentifier;

    /**
     * Default constructor. The id and the Component panel must be set
     * separately.
     */
    public WizardPanelDescriptor() {
        panelIdentifier = DEFAULT_PANEL_IDENTIFIER;
        targetPanel = new JPanel(new BorderLayout());
    }

    /**
     * Constructor which accepts both the Object-based identifier and a
     * reference to the Component class which makes up the panel.
     * @param id Object-based identifier
     * @param panel A class which extends java.awt.Component that will be
     *        inserted as a panel into the wizard dialog.
     */
    public WizardPanelDescriptor(final Object id, final Component panel) {
        panelIdentifier = id;
        targetPanel = panel;
    }

    /**
     * Returns to java.awt.Component that serves as the actual panel.
     * @return A reference to the java.awt.Component that serves as the panel
     */
    public Component getPanelComponent() {
        return targetPanel;
    }

    /**
     * Sets the panel's component as a class that extends java.awt.Component.
     * @param panel java.awt.Component which serves as the wizard panel
     */
    public void setPanelComponent(final Component panel) {
        targetPanel = panel;
    }

    /**
     * Returns the unique Object-based identifier for this panel descriptor.
     * @return The Object-based identifier
     */
    public Object getPanelDescriptorIdentifier() {
        return panelIdentifier;
    }

    /**
     * Sets the Object-based identifier for this panel. The identifier must be
     * unique from all the other identifiers in the panel.
     * @param id Object-based identifier for this panel.
     */
    public void setPanelDescriptorIdentifier(final Object id) {
        panelIdentifier = id;
    }

    /**
     * Set the wizard.
     * @param wizard the wizard new value.
     */
    public void setWizard(final Wizard wizard) {
        this.wizard = wizard;
    }

    /**
     * Returns a reference to the Wizard component.
     * @return The Wizard class hosting this descriptor.
     */
    public Wizard getWizard() {
        return wizard;
    }

    /**
     * Returns a reference to the current WizardModel for this Wizard component.
     * @return The current WizardModel for this Wizard component.
     */
    public WizardModel getWizardModel() {
        return wizard.getModel();
    }

    /**
     * Override this class to provide the Object-based identifier of the panel
     * that the user should traverse to when the Next button is pressed. Note
     * that this method is only called when the button is actually pressed, so
     * that the panel can change the next panel's identifier dynamically at
     * runtime if necessary. Return null if the button should be disabled.
     * Return FinishIdentfier if the button text should change to 'Finish' and
     * the dialog should end.
     * @return Object-based identifier.
     */
    public Object getNextPanelDescriptor() {
        return null;
    }

    /**
     * @return Object-based identifier
     */
    public Object getBackPanelDescriptor() {
        return null;
    }

    /**
     * @return Object-based identifier
     */
    public Object getConfigPanelDescriptor() {
        return null;
    }

    /**
     * @return Object-based identifier
     */
    public Object getPreferencesPanelDescriptor() {
        return null;
    }

    /**
     * @return Object-based identifier
     */
    public Object getStopPanelDescriptor() {
        return null;
    }

    /**
     * Override this method to provide functionality that will be performed just
     * before the panel is to be displayed.
     */
    public void aboutToDisplayPanel() {
    }

    /**
     * Override this method to perform functionality when the panel itself is
     * displayed.
     */
    public void displayingPanel() {
    }

    /**
     * Override this method to perform functionality just before the panel is to
     * be hidden.
     */
    public void aboutToHidePanel() {
    }

    /**
     * FinishIdentifier class.
     */
    public static class FinishIdentifier {
        /**
         * Finish identifier.
         */
        public static final String ID = "FINISH";
    }
}
